@@ -6,6 +6,8 @@ import time
 import random
 import threading
 import json
+import asyncio
+import ssl
 from http.client import HTTPResponse
 
 # Used for global variables
@@ -17,6 +19,7 @@ from stripe._error import APIConnectionError
 from typing import (
     Any,
     Dict,
+    Iterable,
     List,
     Mapping,
     MutableMapping,
@@ -26,11 +29,14 @@ from typing import (
     Union,
     cast,
     overload,
+    AsyncIterable,
 )
 from typing_extensions import (
     Literal,
     NoReturn,
     TypedDict,
+    Awaitable,
+    Never,
 )
 
 # - Requests is the preferred HTTP library
@@ -49,6 +55,22 @@ try:
 except ImportError:
     pycurl = None
 
+try:
+    import httpx
+    import anyio
+    from httpx import Timeout as HTTPXTimeout
+    from httpx import Client as HTTPXClientType
+except ImportError:
+    httpx = None
+    anyio = None
+
+try:
+    import aiohttp
+    from aiohttp import ClientTimeout as AIOHTTPTimeout
+    from aiohttp import StreamReader as AIOHTTPStreamReader
+except ImportError:
+    aiohttp = None
+
 try:
     import requests
     from requests import Session as RequestsSession
@@ -106,6 +128,17 @@ def new_default_http_client(*args: Any, **kwargs: Any) -> "HTTPClient":
     return impl(*args, **kwargs)
 
 
+def new_http_client_async_fallback(*args: Any, **kwargs: Any) -> "HTTPClient":
+    if httpx:
+        impl = HTTPXClient
+    elif aiohttp:
+        impl = AIOHTTPClient
+    else:
+        impl = NoImportFoundAsyncClient
+
+    return impl(*args, **kwargs)
+
+
 class HTTPClient(object):
     name: ClassVar[str]
 
@@ -123,14 +156,13 @@ class HTTPClient(object):
         self,
         verify_ssl_certs: bool = True,
         proxy: Optional[Union[str, _Proxy]] = None,
+        async_fallback_client: Optional["HTTPClient"] = None,
     ):
         self._verify_ssl_certs = verify_ssl_certs
         if proxy:
             if isinstance(proxy, str):
                 proxy = {"http": proxy, "https": proxy}
-            if not isinstance(
-                proxy, dict
-            ):  # pyright: ignore[reportUnnecessaryIsInstance]
+            if not isinstance(proxy, dict):  # pyright: ignore[reportUnnecessaryIsInstance]
                 raise ValueError(
                     "Proxy(ies) must be specified as either a string "
                     "URL or a dict() with string URL under the"
@@ -141,6 +173,7 @@ class HTTPClient(object):
                     " keys."
                 )
         self._proxy = proxy.copy() if proxy else None
+        self._async_fallback_client = async_fallback_client
 
         self._thread_local = threading.local()
 
@@ -258,7 +291,6 @@ class HTTPClient(object):
                 request_id, request_duration_ms, usage=usage
             )
 
-    # TODO: more specific types here would be helpful
     def request_with_retries(
         self,
         method: str,
@@ -365,7 +397,7 @@ class HTTPClient(object):
         headers: Optional[Mapping[str, str]],
         post_data: Any = None,
         *,
-        _usage: Optional[List[str]] = None
+        _usage: Optional[List[str]] = None,
     ) -> Tuple[str, int, Mapping[str, str]]:
         raise NotImplementedError(
             "HTTPClient subclasses must implement `request`"
@@ -378,7 +410,7 @@ class HTTPClient(object):
         headers: Optional[Mapping[str, str]],
         post_data: Any = None,
         *,
-        _usage: Optional[List[str]] = None
+        _usage: Optional[List[str]] = None,
     ) -> Tuple[Any, int, Mapping[str, str]]:
         raise NotImplementedError(
             "HTTPClient subclasses must implement `request_stream`"
@@ -389,6 +421,169 @@ class HTTPClient(object):
             "HTTPClient subclasses must implement `close`"
         )
 
+    async def request_with_retries_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data=None,
+        max_network_retries: Optional[int] = None,
+        *,
+        _usage: Optional[List[str]] = None,
+    ) -> Tuple[Any, int, Any]:
+        return await self._request_with_retries_internal_async(
+            method,
+            url,
+            headers,
+            post_data,
+            is_streaming=False,
+            max_network_retries=max_network_retries,
+            _usage=_usage,
+        )
+
+    async def request_stream_with_retries_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data=None,
+        max_network_retries=None,
+        *,
+        _usage: Optional[List[str]] = None,
+    ) -> Tuple[AsyncIterable[bytes], int, Any]:
+        return await self._request_with_retries_internal_async(
+            method,
+            url,
+            headers,
+            post_data,
+            is_streaming=True,
+            max_network_retries=max_network_retries,
+            _usage=_usage,
+        )
+
+    @overload
+    async def _request_with_retries_internal_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data,
+        is_streaming: Literal[False],
+        max_network_retries: Optional[int],
+        *,
+        _usage: Optional[List[str]] = None,
+    ) -> Tuple[Any, int, Mapping[str, str]]: ...
+
+    @overload
+    async def _request_with_retries_internal_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data,
+        is_streaming: Literal[True],
+        max_network_retries: Optional[int],
+        *,
+        _usage: Optional[List[str]] = None,
+    ) -> Tuple[AsyncIterable[bytes], int, Mapping[str, str]]: ...
+
+    async def _request_with_retries_internal_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data,
+        is_streaming: bool,
+        max_network_retries: Optional[int],
+        *,
+        _usage: Optional[List[str]] = None,
+    ) -> Tuple[Any, int, Mapping[str, str]]:
+        headers = self._add_telemetry_header(headers)
+
+        num_retries = 0
+
+        while True:
+            request_start = _now_ms()
+
+            try:
+                if is_streaming:
+                    response = await self.request_stream_async(
+                        method, url, headers, post_data
+                    )
+                else:
+                    response = await self.request_async(
+                        method, url, headers, post_data
+                    )
+                connection_error = None
+            except APIConnectionError as e:
+                connection_error = e
+                response = None
+
+            if self._should_retry(
+                response, connection_error, num_retries, max_network_retries
+            ):
+                if connection_error:
+                    _util.log_info(
+                        "Encountered a retryable error %s"
+                        % connection_error.user_message
+                    )
+                num_retries += 1
+                sleep_time = self._sleep_time_seconds(num_retries, response)
+                _util.log_info(
+                    (
+                        "Initiating retry %i for request %s %s after "
+                        "sleeping %.2f seconds."
+                        % (num_retries, method, url, sleep_time)
+                    )
+                )
+                await self.sleep_async(sleep_time)
+            else:
+                if response is not None:
+                    self._record_request_metrics(
+                        response, request_start, usage=_usage
+                    )
+
+                    return response
+                else:
+                    assert connection_error is not None
+                    raise connection_error
+
+    async def request_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple[bytes, int, Mapping[str, str]]:
+        if self._async_fallback_client is not None:
+            return await self._async_fallback_client.request_async(
+                method, url, headers, post_data
+            )
+        raise NotImplementedError(
+            "HTTPClient subclasses must implement `request_async`"
+        )
+
+    async def request_stream_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple[AsyncIterable[bytes], int, Mapping[str, str]]:
+        if self._async_fallback_client is not None:
+            return await self._async_fallback_client.request_stream_async(
+                method, url, headers, post_data
+            )
+        raise NotImplementedError(
+            "HTTPClient subclasses must implement `request_stream_async`"
+        )
+
+    async def close_async(self):
+        if self._async_fallback_client is not None:
+            return await self._async_fallback_client.close_async()
+        raise NotImplementedError(
+            "HTTPClient subclasses must implement `close_async`"
+        )
+
+    def sleep_async(self, secs: float) -> Awaitable[None]:
+        if self._async_fallback_client is not None:
+            return self._async_fallback_client.sleep_async(secs)
+        raise NotImplementedError(
+            "HTTPClient subclasses must implement `sleep`"
+        )
+
 
 class RequestsClient(HTTPClient):
     name = "requests"
@@ -399,10 +594,13 @@ class RequestsClient(HTTPClient):
         session: Optional["RequestsSession"] = None,
         verify_ssl_certs: bool = True,
         proxy: Optional[Union[str, HTTPClient._Proxy]] = None,
-        **kwargs
+        async_fallback_client: Optional[HTTPClient] = None,
+        **kwargs,
     ):
         super(RequestsClient, self).__init__(
-            verify_ssl_certs=verify_ssl_certs, proxy=proxy
+            verify_ssl_certs=verify_ssl_certs,
+            proxy=proxy,
+            async_fallback_client=async_fallback_client,
         )
         self._session = session
         self._timeout = timeout
@@ -440,8 +638,7 @@ class RequestsClient(HTTPClient):
         headers: Optional[Mapping[str, str]],
         post_data,
         is_streaming: Literal[True],
-    ) -> Tuple[Any, int, Mapping[str, str]]:
-        ...
+    ) -> Tuple[Any, int, Mapping[str, str]]: ...
 
     @overload
     def _request_internal(
@@ -451,8 +648,7 @@ class RequestsClient(HTTPClient):
         headers: Optional[Mapping[str, str]],
         post_data,
         is_streaming: Literal[False],
-    ) -> Tuple[bytes, int, Mapping[str, str]]:
-        ...
+    ) -> Tuple[bytes, int, Mapping[str, str]]: ...
 
     def _request_internal(
         self,
@@ -583,9 +779,12 @@ class UrlFetchClient(HTTPClient):
         verify_ssl_certs: bool = True,
         proxy: Optional[HTTPClient._Proxy] = None,
         deadline: int = 55,
+        async_fallback_client: Optional[HTTPClient] = None,
     ):
         super(UrlFetchClient, self).__init__(
-            verify_ssl_certs=verify_ssl_certs, proxy=proxy
+            verify_ssl_certs=verify_ssl_certs,
+            proxy=proxy,
+            async_fallback_client=async_fallback_client,
         )
 
         # no proxy support in urlfetch. for a patch, see:
@@ -627,8 +826,7 @@ class UrlFetchClient(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[True],
-    ) -> Tuple[BytesIO, int, Any]:
-        ...
+    ) -> Tuple[BytesIO, int, Any]: ...
 
     @overload
     def _request_internal(
@@ -638,8 +836,7 @@ class UrlFetchClient(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[False],
-    ) -> Tuple[str, int, Any]:
-        ...
+    ) -> Tuple[str, int, Any]: ...
 
     def _request_internal(
         self,
@@ -713,9 +910,12 @@ class PycurlClient(HTTPClient):
         self,
         verify_ssl_certs: bool = True,
         proxy: Optional[HTTPClient._Proxy] = None,
+        async_fallback_client: Optional[HTTPClient] = None,
     ):
         super(PycurlClient, self).__init__(
-            verify_ssl_certs=verify_ssl_certs, proxy=proxy
+            verify_ssl_certs=verify_ssl_certs,
+            proxy=proxy,
+            async_fallback_client=async_fallback_client,
         )
 
         assert pycurl is not None
@@ -762,8 +962,7 @@ class PycurlClient(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[True],
-    ) -> Tuple[BytesIO, int, Any]:
-        ...
+    ) -> Tuple[BytesIO, int, Any]: ...
 
     @overload
     def _request_internal(
@@ -773,8 +972,7 @@ class PycurlClient(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[False],
-    ) -> Tuple[str, int, Mapping[str, str]]:
-        ...
+    ) -> Tuple[str, int, Mapping[str, str]]: ...
 
     def _request_internal(
         self,
@@ -902,9 +1100,12 @@ class Urllib2Client(HTTPClient):
         self,
         verify_ssl_certs: bool = True,
         proxy: Optional[HTTPClient._Proxy] = None,
+        async_fallback_client: Optional[HTTPClient] = None,
     ):
         super(Urllib2Client, self).__init__(
-            verify_ssl_certs=verify_ssl_certs, proxy=proxy
+            verify_ssl_certs=verify_ssl_certs,
+            proxy=proxy,
+            async_fallback_client=async_fallback_client,
         )
         # prepare and cache proxy tied opener here
         self._opener = None
@@ -938,8 +1139,7 @@ class Urllib2Client(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[False],
-    ) -> Tuple[str, int, Any]:
-        ...
+    ) -> Tuple[str, int, Any]: ...
 
     @overload
     def _request_internal(
@@ -949,8 +1149,7 @@ class Urllib2Client(HTTPClient):
         headers: Mapping[str, str],
         post_data,
         is_streaming: Literal[True],
-    ) -> Tuple[HTTPResponse, int, Any]:
-        ...
+    ) -> Tuple[HTTPResponse, int, Any]: ...
 
     def _request_internal(
         self,
@@ -1005,3 +1204,311 @@ class Urllib2Client(HTTPClient):
 
     def close(self):
         pass
+
+
+class HTTPXClient(HTTPClient):
+    name = "httpx"
+
+    _client: Optional["HTTPXClientType"]
+
+    def __init__(
+        self,
+        timeout: Optional[Union[float, "HTTPXTimeout"]] = 80,
+        allow_sync_methods=False,
+        **kwargs,
+    ):
+        super(HTTPXClient, self).__init__(**kwargs)
+
+        if httpx is None:
+            raise ImportError(
+                "Unexpected: tried to initialize HTTPXClient but the httpx module is not present."
+            )
+
+        if anyio is None:
+            raise ImportError(
+                "Unexpected: tried to initialize HTTPXClient but the anyio module is not present."
+            )
+
+        self.httpx = httpx
+        self.anyio = anyio
+
+        kwargs = {}
+        if self._verify_ssl_certs:
+            kwargs["verify"] = stripe.ca_bundle_path
+        else:
+            kwargs["verify"] = False
+
+        self._client_async = httpx.AsyncClient(**kwargs)
+        self._client = None
+        if allow_sync_methods:
+            self._client = httpx.Client(**kwargs)
+        self._timeout = timeout
+
+    def sleep_async(self, secs):
+        return self.anyio.sleep(secs)
+
+    def _get_request_args_kwargs(
+        self, method: str, url: str, headers: Mapping[str, str], post_data
+    ):
+        kwargs = {}
+
+        if self._proxy:
+            kwargs["proxies"] = self._proxy
+
+        if self._timeout:
+            kwargs["timeout"] = self._timeout
+        return [
+            (method, url),
+            {"headers": headers, "data": post_data or {}, **kwargs},
+        ]
+
+    def request(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data=None,
+    ) -> Tuple[bytes, int, Mapping[str, str]]:
+        if self._client is None:
+            raise RuntimeError(
+                "Stripe: HTTPXClient was initialized with allow_sync_methods=False, "
+                "so it cannot be used for synchronous requests."
+            )
+        args, kwargs = self._get_request_args_kwargs(
+            method, url, headers, post_data
+        )
+        try:
+            response = self._client.request(*args, **kwargs)
+        except Exception as e:
+            self._handle_request_error(e)
+
+        content = response.content
+        status_code = response.status_code
+        response_headers = response.headers
+        return content, status_code, response_headers
+
+    async def request_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data=None,
+    ) -> Tuple[bytes, int, Mapping[str, str]]:
+        args, kwargs = self._get_request_args_kwargs(
+            method, url, headers, post_data
+        )
+        try:
+            response = await self._client_async.request(*args, **kwargs)
+        except Exception as e:
+            self._handle_request_error(e)
+
+        content = response.content
+        status_code = response.status_code
+        response_headers = response.headers
+        return content, status_code, response_headers
+
+    def _handle_request_error(self, e) -> NoReturn:
+        msg = (
+            "Unexpected error communicating with Stripe. If this "
+            "problem persists, let us know at support@stripe.com."
+        )
+        err = "A %s was raised" % (type(e).__name__,)
+        should_retry = True
+
+        msg = textwrap.fill(msg) + "\n\n(Network error: %s)" % (err,)
+        raise APIConnectionError(msg, should_retry=should_retry)
+
+    def request_stream(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple[Iterable[bytes], int, Mapping[str, str]]:
+        if self._client is None:
+            raise RuntimeError(
+                "Stripe: HTTPXClient was not initialized with allow_sync_methods=True, "
+                "so it cannot be used for synchronous requests."
+            )
+        args, kwargs = self._get_request_args_kwargs(
+            method, url, headers, post_data
+        )
+        try:
+            response = self._client.send(
+                request=self._client_async.build_request(*args, **kwargs),
+                stream=True,
+            )
+        except Exception as e:
+            self._handle_request_error(e)
+        content = response.iter_bytes()
+        status_code = response.status_code
+        headers = response.headers
+
+        return content, status_code, headers
+
+    async def request_stream_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple[AsyncIterable[bytes], int, Mapping[str, str]]:
+        args, kwargs = self._get_request_args_kwargs(
+            method, url, headers, post_data
+        )
+        try:
+            response = await self._client_async.send(
+                request=self._client_async.build_request(*args, **kwargs),
+                stream=True,
+            )
+        except Exception as e:
+            self._handle_request_error(e)
+        content = response.aiter_bytes()
+        status_code = response.status_code
+        headers = response.headers
+
+        return content, status_code, headers
+
+    def close(self):
+        if self._client is not None:
+            self._client.close()
+
+    async def close_async(self):
+        await self._client_async.aclose()
+
+
+class AIOHTTPClient(HTTPClient):
+    name = "aiohttp"
+
+    def __init__(
+        self, timeout: Optional[Union[float, "AIOHTTPTimeout"]] = 80, **kwargs
+    ):
+        super(AIOHTTPClient, self).__init__(**kwargs)
+
+        if aiohttp is None:
+            raise ImportError(
+                "Unexpected: tried to initialize AIOHTTPClient but the aiohttp module is not present."
+            )
+
+        self._timeout = timeout
+        self._cached_session = None
+
+    @property
+    def _session(self):
+        assert aiohttp is not None
+
+        if self._cached_session is None:
+            kwargs = {}
+            if self._verify_ssl_certs:
+                ssl_context = ssl.create_default_context(
+                    cafile=stripe.ca_bundle_path
+                )
+                kwargs["connector"] = aiohttp.TCPConnector(ssl=ssl_context)
+            else:
+                kwargs["connector"] = aiohttp.TCPConnector(verify_ssl=False)
+
+            self._cached_session = aiohttp.ClientSession(**kwargs)
+
+        return self._cached_session
+
+    def sleep_async(self, secs):
+        return asyncio.sleep(secs)
+
+    def request(self) -> Tuple[bytes, int, Mapping[str, str]]:
+        raise NotImplementedError(
+            "AIOHTTPClient does not support synchronous requests."
+        )
+
+    def _get_request_args_kwargs(
+        self, method: str, url: str, headers: Mapping[str, str], post_data
+    ):
+        args = (method, url)
+        kwargs = {}
+        if self._proxy:
+            if self._proxy["http"] != self._proxy["https"]:
+                raise ValueError(
+                    "AIOHTTPClient does not support different proxies for HTTP and HTTPS."
+                )
+            kwargs["proxy"] = self._proxy["https"]
+        if self._timeout:
+            kwargs["timeout"] = self._timeout
+
+        kwargs["headers"] = headers
+        kwargs["data"] = post_data
+        return args, kwargs
+
+    async def request_async(
+        self,
+        method: str,
+        url: str,
+        headers: Mapping[str, str],
+        post_data=None,
+    ) -> Tuple[bytes, int, Mapping[str, str]]:
+        (
+            content,
+            status_code,
+            response_headers,
+        ) = await self.request_stream_async(
+            method, url, headers, post_data=post_data
+        )
+
+        return (await content.read()), status_code, response_headers
+
+    def _handle_request_error(self, e) -> NoReturn:
+        msg = (
+            "Unexpected error communicating with Stripe. If this "
+            "problem persists, let us know at support@stripe.com."
+        )
+        err = "A %s was raised" % (type(e).__name__,)
+        should_retry = True
+
+        msg = textwrap.fill(msg) + "\n\n(Network error: %s)" % (err,)
+        raise APIConnectionError(msg, should_retry=should_retry)
+
+    def request_stream(self) -> Tuple[Iterable[bytes], int, Mapping[str, str]]:
+        raise NotImplementedError(
+            "AIOHTTPClient does not support synchronous requests."
+        )
+
+    async def request_stream_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple["AIOHTTPStreamReader", int, Mapping[str, str]]:
+        args, kwargs = self._get_request_args_kwargs(
+            method, url, headers, post_data
+        )
+        try:
+            response = await self._session.request(*args, **kwargs)
+        except Exception as e:
+            self._handle_request_error(e)
+
+        content = response.content
+        status_code = response.status
+        response_headers = response.headers
+        return content, status_code, response_headers
+
+    def close(self):
+        pass
+
+    async def close_async(self):
+        await self._session.close()
+
+
+class NoImportFoundAsyncClient(HTTPClient):
+    def __init__(self, **kwargs):
+        super(NoImportFoundAsyncClient, self).__init__(**kwargs)
+
+    @staticmethod
+    def raise_async_client_import_error() -> Never:
+        raise ImportError(
+            (
+                "Import httpx not found. To make async http requests,"
+                "You must either install httpx or define your own"
+                "async http client by subclassing stripe.HTTPClient"
+                "and setting stripe.default_http_client to an instance of it."
+            )
+        )
+
+    async def request_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ) -> Tuple[bytes, int, Mapping[str, str]]:
+        self.raise_async_client_import_error()
+
+    async def request_stream_async(
+        self, method: str, url: str, headers: Mapping[str, str], post_data=None
+    ):
+        self.raise_async_client_import_error()
+
+    async def close_async(self):
+        self.raise_async_client_import_error()
